#!/bin/sh
# Copyright (c) 2017 Intel Corporation. All rights reserved.

# This script is used to generate a 'make dist' style tarball that can then be
# used to generate rpms using rpmbuild -ta.

VERSION=$(grep "MVAPICH2_VERSION_m4" maint/version.m4 | grep -o -E "[0-9]*\.[0-9]*[a-z]?")
RELEASE="release"
TAG="wfr-orig-$VERSION"

if [ -e .git ]; then
	RELEASE=`git describe --tags --long --match="$TAG" | sed -e "s/$TAG-//" -e 's/-g.*$//'`
fi

echo "VERSION $VERSION"
echo "RELEASE $RELEASE"

echo "Creating distribution directory..."
TARBALL_DIR=mvapich2-$VERSION-$RELEASE
mkdir .$TARBALL_DIR
cp -a * .$TARBALL_DIR
mv .$TARBALL_DIR $TARBALL_DIR

sed -e "s/@VERSION@/$VERSION/g" -e "s/@RELEASE@/$RELEASE/g" mvapich2.spec.in > $TARBALL_DIR/mvapich2.spec
git log --no-merges $TAG..HEAD --format="* %cd <%ae>%n- %s%n" \
	| sed 's/-[0-9][0-9][0-9][0-9] //' \
	| sed 's/ [0-9][0-9]:[0-9][0-9]:[0-9][0-9]//' \
	>> $TARBALL_DIR/mvapich2.spec
cp $TARBALL_DIR/mvapich2.spec .

# LIMIC includes a second spec file, which creates problems with rpmbuild -ta.
# LIMIC isn't used, so just remove it.
rm -rf $TARBALL_DIR/limic2-*

# Run autogen.sh with automake 2.15 and libtool 2.4.3 or newer
# so that the built srpm can be built with older tools.
# However, this script is also called by the cvs import script.
# To avoid importing all the generated files, comment it out here.
# If one wishes to generate srpm that can be rebuilt with old automake
# tool through the makesrpm.sh script, uncomment it.
#cd $TARBALL_DIR
#./autogen.sh
#cd ..

echo "Creating tarball..."
tar czf $TARBALL_DIR.tar.gz $TARBALL_DIR

echo "Cleaning up..."
rm -rf $TARBALL_DIR

